/***************************************************************************/
/*                                                                         */
/*   sun_posi.c :uÓVwvLڂ̕@ɂ鑾z̈ʒǔvZ              */
/*      Ver. 1.2    1999/07/02          By   takesako@mrj.biglobe.ne.jp    */
/*                                           (c)1999 Shinobu Takesako      */
/***************************************************************************/
/*   1999/06/29   planet_posi.c̕                                  */
/*                Solar_Geo()̒ǉ                                        */
/*   1999/07/02   Solar_Geo()̃oOC                                    */
/***************************************************************************/

#include <stdio.h>
#include <math.h>

#include "astro.h"
#include "Tafel_VIII.h"

void Sun_Position(double jd , Position *Sun)
{
	double c;
	double sml,spl,sec,sax,sma,smpg,sta,sl,srr,ss;
	double sx,sy,sz;
	double Ra, Dec;

    c    = C_1900(jd);

	sml  = 280.6824 + 36000.769325*c + 7.22222e-4*c*c;
	sml  = Compact (sml,360);

	spl  = 281.2206 + 1.717697*c + 4.83333e-4*c*c + 2.77777e-6*c*c*c;
	spl  = Compact (spl , 360);

	sec  = 0.0167498 - 4.258e-5*c - 1.37e-7*c*c;
	sax  = 1.00000129;

	sma  = sml - spl;
	smpg = 1.91946*sin(sma*pai) + 2.00939e-2*sin(2*sma*pai) - 4.78889e-3*sin(sma*pai)*c -1.44444e-5*sin(sma*pai)*c*c;
	sl   = sml + smpg;
	sta  = sl - spl;
	srr  = sax * ( 1 - sec*sec )/( 1 + sec*cos(sta*pai));
	ss   = 0.2666/ srr;

	sx   = srr*cos(sl*pai);
	sy   = srr*sin(sl*pai);
	sz   = 0;

/*	printf("\n\n sml= %.6f spl= %.6f sec= %.6f sax= %.6f\n sma= %.6f smpg= %.6f sta= %.6f sl= %.6f\n srr= %.6f,ss= %.6f sx= %.6f sy= %.6f sz= %.6f\n\n",sml,spl,sec,sax,sma,smpg,sta,sl,srr,ss,sx,sy,sz);
*/
	Sun->lam = sl;
	Sun->bet = 0.0;
	Sun->r   = srr;
	Sun->x   = sx;
	Sun->y   = sy;
	Sun->z   = sz;
	Sun->ss  = ss;
	LamToRa( jd, sl, 0.0, &Ra, &Dec);
	Sun->ra  = Ra;
	Sun->dec = Dec;
}


/*******************************************************************************/
/*                 NiYear0jɂ鑾z̒nSԓW̌vZ           */
/*                  (Astronomical algorithmsɂ)                            */
/*******************************************************************************/

void Solar_Geo (double jd, double Year0,double *X,double *Y,double *Z)
{
	double jd0,t,T, Zeta,Zeta1,Zeta2,Zeta3,Theta,Theta1,Theta2,Theta3,Zed,Zed1,Zed2,Zed3;
	double cosTheta,sinTheta ,cosZed,sinZed,cosZeta ,sinZeta;
	double Xx,Xy,Xz,Yx,Yy,Yz,Zx,Zy,Zz,X0,Y0,Z0, obl,cosOBL,sinOBL;
	
	jd0=JD( Year0,1,1,12,0,0); /*1999/07/02*/

	T=(jd0-2451545.0)/36525.0; /*1999/07/02*/
	t=-(jd-jd0)/36525.0;

	Zeta1  = 2306.2181 + 1.39656*T - 0.000139*T*T;
	Zeta2  = 0.30188 - 0.000344*T;
	Zeta3  = 0.017998;
	Theta1 = 2004.3109 - 0.85330*T - 0.000217*T*T;
	Theta2 = - ( 0.42665 + 0.000217*T);
	Theta3 = - 0.041833;
	Zed1   = 2306.2181 + 1.39656*T - 0.000139*T*T;
	Zed2   = 1.09468 + 0.000066*T;
	Zed3   = 0.018203;

	Zeta  = (  Zeta1*t +  Zeta2*t*t + Zeta3*t*t*t )/3600.0;
	Zed   = (   Zed1*t +   Zed2*t*t +  Zed3*t*t*t )/3600.0;
	Theta = ( Theta1*t + Theta2*t*t +Theta3*t*t*t )/3600.0;

/*printf("Zeta=%10.6f Zed =%10.6f Theta=%10.6f \n",Zeta,Zed,Theta);*/

	cosTheta  = cos (Theta*pai);
	sinTheta  = sin (Theta*pai);
	cosZed   = cos (Zed*pai);
	sinZed   = sin (Zed*pai);
	cosZeta = cos(Zeta*pai);
	sinZeta = sin(Zeta*pai);

    Xx = cosZeta*cosZed*cosTheta - sinZeta*sinZed;
    Xy = sinZeta*cosZed + cosZeta*sinZed*cosTheta;
    Xz =                  cosZeta*sinTheta;
    
    Yx = -cosZeta*sinZed - sinZeta*cosZed*cosTheta;
    Yy =  cosZeta*cosZed - sinZeta*sinZed*cosTheta;
    Yz =                 - sinZeta*sinTheta;
    
    Zx = -cosZed*sinTheta;
    Zy = -sinZed*sinTheta;
    Zz =         cosTheta;

    Sun_Position (jd, &Sun );

	obl     = Obl(jd);
	cosOBL  = cos(obl*pai);
	sinOBL  = sin(obl*pai);

	X0 = Sun.r * cos(Sun.lam*pai);
	Y0 = Sun.r * sin(Sun.lam*pai) * cosOBL;
	Z0 = Sun.r * sin(Sun.lam*pai) * sinOBL;

/*printf("X0 =%12.6f Y0 =%12.6f Z0 =%12.6f \n",X0,Y0,Z0);*/
	
	*X = Xx*X0 + Yx*Y0 + Zx*Z0;
	*Y = Xy*X0 + Yy*Y0 + Zy*Z0;
	*Z = Xz*X0 + Yz*Y0 + Zz*Z0;
/*printf("X  =%12.6f Y  =%12.6f Z  =%12.6f \n",*X,*Y,*Z);*/

}


